Package: Kira
Type: Package
Title: Machine Learning
Version: 1.0.8
Date: 2026-04-13
Imports: graphics, grDevices, MASS, stats
Authors@R: c(person(given = c("Paulo Cesar"), family = "Ossani", 
             email = "ossanipc@hotmail.com", 
             role = c("aut", "cre"), 
             comment = c(ORCID = "0000-0002-6617-8085")))
Depends: R (>= 3.3.2)
Description: Machine learning, containing several algorithms for supervised and unsupervised classification, in addition to a function that plots the Receiver Operating Characteristic (ROC) and Precision-Recall (PRC) curve graphs, and also a function that returns several metrics used for model evaluation, the latter can be used in ranking results from other packs.
License: GPL-3
Encoding: UTF-8
NeedsCompilation: yes
Author: Paulo Cesar Ossani [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6617-8085>)
Maintainer: Paulo Cesar Ossani <ossanipc@hotmail.com>
Repository: CRAN
Packaged: 2026-04-13 18:31:17 UTC; sec-l
Date/Publication: 2026-04-13 19:30:10 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 20:47:00 UTC; unix
Archs: Kira.so.dSYM
